searchNodes=[{"doc":"Loads dynamic configuration at runtime. Why Vapor? Dynamically configuring elixir apps can be hard. There are major differences between configuring applications with mix and configuring applications in a release. Vapor wants to make all of that easy by providing an alternative to mix config for runtime configs. Specifically Vapor can: Find and load configuration from files (JSON, YAML, TOML). Read configuration from environment variables. .env file support for easy local development. Example defmodule VaporExample.Application do use Application alias Vapor.Provider.{File, Env} def start(_type, _args) do providers = [ %Env{bindings: [db_url: &quot;DB_URL&quot;, db_name: &quot;DB_NAME&quot;, port: &quot;PORT&quot;]}, %File{path: &quot;config.toml&quot;, bindings: [kafka_brokers: &quot;kafka.brokers&quot;]}, ] # If values could not be found we raise an exception and halt the boot # process config = Vapor.load!(providers) children = [ {VaporExampleWeb.Endpoint, port: config.port} {VaporExample.Repo, [db_url: config.db_url, db_name: config.db_name]}, {VaporExample.Kafka, brokers: config.kafka_brokers}, ] opts = [strategy: :one_for_one, name: VaporExample.Supervisor] Supervisor.start_link(children, opts) end end Precedence Vapor merges the configuration based on the order that the providers are specified. providers = [ %Dotenv{}, %File{path: &quot;$HOME/.vapor/config.json&quot;, bindings: []}, %Env{bindings: []}, ] Env will have the highest precedence, followed by File, and finally Dotenv. Translating config values Its often useful to translate configuration values into something meaningful for your application. Vapor provides translation options to allow you to do any type conversions or data manipulation. providers = [ %Env{bindings: [db_name: &quot;DB_NAME&quot;, port: &quot;PORT&quot;]}, %File{path: &quot;config.toml&quot;, bindings: [kafka_brokers: &quot;kafka.brokers&quot;]}, ] translations = [ port: fn s -&gt; String.to_integer(s) end, kafka_brokers: fn str -&gt; str |&gt; String.split(&quot;,&quot;) |&gt; Enum.map(fn broker -&gt; [host, port] = String.split(broker, &quot;:&quot;) {host, String.to_integer(port)} end) end ] config = Vapor.load(providers, translations) Reading config files Config files can be read from a number of different file types including JSON, TOML, and YAML. Vapor determines which file format to use based on the file extension. Options on bindings Bindings for %Env{} and %File{} providers support a number of options: :map - Allows you to pass a &quot;translation&quot; function with the binding. :default - If the value is not found then the default value will be returned instead. Defaults always skip the translations. :required - Marks the binding a required or not required (defaults to true). If required values are missing, and there is no default present, then the provider will return an exception. If the binding is marked required: false, then the provider returns the key with a nil value. providers = [ %Env{ bindings: [ {:db_name, &quot;DB_NAME&quot;}, {:db_port, &quot;DB_PORT&quot;, default: 4369, map: &amp;String.to_integer/1}, ] } ] Overriding application environment In some cases you may want to overwrite the keys in the application environment for convenience. While this is generally discouraged it can be a quick way to adopt Vapor. Before you start your supervision tree you can use something like this: config = Vapor.load!(providers) Application.put_env(:my_app, MyApp.Repo, [ database: config[:database], username: config[:database_user], password: config[:database_pasword], hostname: config[:database_host], port: config[:database_port], ]) Providers There are several built in providers Environment .env files JSON YAML TOML If you need to create a new provider you can do so with the included Vapor.Provider protocol. defmodule MyApp.DatabaseProvider do defstruct [id: nil] defimpl Vapor.Provider do def load(db_provider) do end end end","ref":"Vapor.html","title":"Vapor","type":"module"},{"doc":"Loads a configuration plan.","ref":"Vapor.html#load/2","title":"Vapor.load/2","type":"function"},{"doc":"Loads a configuration plan or raises","ref":"Vapor.html#load!/2","title":"Vapor.load!/2","type":"function"},{"doc":"","ref":"Vapor.Provider.html","title":"Vapor.Provider","type":"protocol"},{"doc":"The protocol required for loading a configuration. Providers need to implement this protocol and must return either an {:ok, %{}} or {:error, term()}.","ref":"Vapor.Provider.html#load/1","title":"Vapor.Provider.load/1","type":"function"},{"doc":"","ref":"Vapor.Provider.html#t:t/0","title":"Vapor.Provider.t/0","type":"type"},{"doc":"The dotenv config provider will look for a .env file and load all of the values for that file. The values can be written like so: DATABASE_URL=https://localhost:9432 PORT=4000 REDIS_HOST=1234 If the file can&#39;t be found then this provider will still return an ok but will (obviously) not load any configuration values. The primary use case for this provider is local development where it might be inconvenient to add all of the necessary environment variables on your local machine and it makes tradeoffs for that use case. Existing environment variables By default the dotenv provider won&#39;t overwrite any existing environment variables. You can change this by setting the overwrite key to true: %Dotenv{overwrite: true} File heirarchy If no file is specified then the dotenv provider will load these files in this order. Each proceeding file is loaded over the previous. In these examples ENV will be the current mix environment: dev, test, or prod. .env .env.ENV .env.local .env.ENV.local You should commit .env and .env.ENV files to your project and ignore any .local files. This allows users to provide a custom setup if they need to do that.","ref":"Vapor.Provider.Dotenv.html","title":"Vapor.Provider.Dotenv","type":"module"},{"doc":"The Env config module provides support for pulling configuration values from the environment. Bindings must be specified at a keyword list. Example %Env{bindings: [foo: &quot;FOO&quot;, bar: &quot;VAR_BAR&quot;]}","ref":"Vapor.Provider.Env.html","title":"Vapor.Provider.Env","type":"module"},{"doc":"Module for loading supported file format configs Supported file formats: .json, .toml, .yaml. Bindings to specific keys must be provided as a keyword list. The values for each key must be either a string or a path based on the Access protocol. Example %File{path: &quot;config.toml&quot;, bindings: [foo: &quot;foo&quot;, nested: [&quot;some&quot;, &quot;nested&quot;, &quot;value&quot;]]}","ref":"Vapor.Provider.File.html","title":"Vapor.Provider.File","type":"module"},{"doc":"","ref":"Vapor.Provider.File.html#s/0","title":"Vapor.Provider.File.s/0","type":"function"},{"doc":"Allows users to group together bits of configuration. This allows users to avoid duplication and avoids conflicts in common names such as &quot;port&quot; and &quot;host&quot;. Example providers = [ %Group{ name: :primary_db, providers: [ %Env{bindings: [port: &quot;PRIMARY_DB_PORT&quot;, host: &quot;PRIMARY_DB_HOST&quot;]}, ] }, %Group{ name: :redis, providers: [ %Env{bindings: [port: &quot;REDIS_PORT&quot;, host: &quot;REDIS_HOST&quot;]}, ] }, ]","ref":"Vapor.Provider.Group.html","title":"Vapor.Provider.Group","type":"module"},{"doc":"","ref":"Vapor.FileFormatNotFoundError.html","title":"Vapor.FileFormatNotFoundError","type":"exception"},{"doc":"","ref":"Vapor.FileNotFoundError.html","title":"Vapor.FileNotFoundError","type":"exception"},{"doc":"","ref":"Vapor.LoadError.html","title":"Vapor.LoadError","type":"exception"}]